# COVID testy

Na Gamči začala nová vlna covidu. Aby sa predišlo ďalšiemu šíreniu nákazy, Gamča sa rozhodla všetkých študentov otestovať antigénovými testami zo slín.

Testy sú však celkom drahé, takže Gamča by rada použila čo najmenej testov. Je to trochu nechutné, ale je možné zmiešať vzorky od viacerých študentov a otestovať túto novú zmiešanú vzorku jedným testom. Ak test vyjde negatívny, budú tak vedieť, že všetky pôvodné vzorky sú negatívne. Ak vyjde pozitívny, budú vedieť, že aspoň jedna z pôvodných vzoriek bola pozitívna. Toto by mohli využiť na zníženie počtu použitých testov.

Škola dostala vzorky slín od $N$ študentov (kvôli GDPR nevedia ich mená, len čísla $0$ až $N-1$) a potrebujú zistiť, ktoré vzorky sú pozitívne a ktoré negatívne. Z lokálnych štatistík vedia, že pravdepodobnosť, že konkrétny študent je nakazený, je $P$. To, či je nejaká vzorka pozitívna alebo negatívna, nemá žiaden vplyv na pravdepodobnosť pozitívnosti žiadnej inej vzorky.

V každej vzorke je dosť slín na to, aby sa dala testovať ľubovoľne veľa krát (a byť použitá ľubovoľne veľa krát pre zmiešané vzorky). Testy tiež majú perfektnú spoľahlivosť – vždy dajú pre vzorku rovnaký výsledok, výsledok je vždy pozitívny, ak test obsahoval sliny nakazeného študenta, a je vždy negatívny, ak neobsahoval sliny žiadneho nakazeného študenta.

Za týchto podmienok by potrebovali optimalizovať testovanie tak, aby použili čo najmenej testov. Žiaľ, väčšina profesorov je tiež doma s covidom, čiže to ostáva na vás.

## Komunikácia

Táto úloha je interaktívna.

Váš program bude pustený na viacerých vstupoch. Pre jeden vstup, a teda jeden beh vášho programu, budete potrebovať vyriešiť $T$ rôznych scenárov. Hodnoty $N$ a $P$ sú rovnaké vo všetkých scenároch, ale to, ktoré vzorky sú pozitívne, sa bude (pravdepodobne) líšiť medzi scenármi.

Protokol pre komunikáciu s testovačom je implementovaný v prílohe `template.cpp`. Ak preferujete, môžete si napísať vlastnú implementáciu. Pre podrobnosti o oboch spôsoboch viď príslušné podsekcie nižšie.

### Poskytnutá implementácia (template)

Ak používate implementáciu protokolu z `template.cpp`, musíte implementovať funkciu `std::vector<bool> find_positive()`. Táto funkcia sa zavolá raz pre každý scenár. Musí vrátiť vašu odpoveď ako vektor pravdivostných hodnôt dĺžky $N$, kde $i$-ty prvok je `true`, ak je vzorka študenta s číslom $i$ pozitívna, inak je `false`.

Môžete využiť funkciu `bool test_students(std::vector<bool> mask)` na zistenie, ktoré vzorky sú pozitívne. Táto funkcia otestuje jednu vzorku (môže byť zmiešaná z ľubovoľnej podmnožiny pôvodných vzoriek). Jej jediný argument je vektor pravdivostných hodnôt dĺžky $N$, kde $i$-ty prvok je `true`, ak je vzorka študenta s číslom $i$ zahrnutá v testovanej vzorke, inak je `false`.
Funkcia vráti `true`, ak aspoň jedna zo zahrnutých pôvodných vzoriek je pozitívna, inak vráti `false`.

Môžete tiež využívať globálne premenné `N` a `P` obsahujúce $N$ a $P$ zo zadania.
Akúkoľvek inicializáciu môžete robiť vo funkcii `main` po prvom zavolaní `scanf`.

### Protokol

Ak použijete poskytnutý template, nasledujúci protokol je implementovaný za vás.

Na začiatku by váš program mal prečítať riadok zo štandardného vstupu obsahujúci celé číslo $N$,
reálne číslo $P$ a celé číslo $T$ oddelené medzerami —
počet študentov, pravdepodobnosť, že vzorka je pozitívna, a počet scenárov.

Následne program môže písať otázky na štandardný výstup. Každá otázka by mal byť jeden riadok obsahujúci písmeno `Q`, medzeru, a reťazec $s$ dĺžky $N$, kde $s_i$ je `1`, ak $i$-ta vzorka má byť zahrnutá v testovanej vzorke, a `0`, ak nie.
Po vypísaní tohto riadku treba spláchnuť (*flush*) štandardný výstup a potom prečítať jeden riadok, na ktorom je jediný znak — `P`, ak je test pozitívny, inak `N`.

Program tiež môže na štandardný výstup vypísať odpoveď ako jeden riadok obsahujúci písmeno `A`, medzeru, a reťazec $s$ dĺžky $N$, kde $s_i$ je `1`, ak $i$-ta vzorka je pozitívna, a `0`, ak nie. Po vypísaní tohto riadku treba spláchnuť štandardný výstup a potom prečítať jeden riadok, na ktorom je jediný znak.

Ak je tento znak `C`, vaša odpoveď bola správna. Váš program môže začať klásť otázky k ďalšiemu scenáru, alebo, ak toto bola vaša $T$-ta odpoveď, skončiť.

Ak je tento znak `W`, vaša odpoveď bola nesprávna. V takom prípade má váš program okamžite skončiť.

Upozorňujeme, že ukončenie programu po načítaní `W` je dôležité na získanie zmysluplného ohodnotenia od testovača. Ak váš program bude pokračovať, môže nečakane skončiť (kvôli načítaniu ukončeného vstupu) alebo dostať iné neúspešné ohodnotenie.

## Vstupy

Testovač pre túto úlohu nie je adaptívny,
teda pozitívnosť jednotlivých vzoriek je určená
pred spustením programu a počas jeho behu sa nemení.
Navyše, to, či je daná vzorka pozitívna,
je určené nezávisle od ostatných vzoriek s pravdepodobnosťou $P$
pomocou spravodlivého generátora náhodných čísel.

## Podúlohy a bodovanie

Sú dve podúlohy.

### Prvá podúloha (10 bodov)

- $N = 1\,000$
- $T = 1$
- $0 \le P \le 1$

Riešenie je akceptované, ak odpovie správne a použije najviac $2 \cdot N$ otázok pre každý vstup.

### Druhá podúloha (90 bodov)

- $N = 1\,000$
- $T = 300$
- $0.001 \le P \le 0.2$

Táto podúloha používa čiastkové bodovanie (partial scoring).

Ak bude vaša odpoveď na akýkoľvek scenár nesprávna, dostanete nula bodov.

V opačnom prípade sa počet bodov za daný vstup určí
na základe priemerného počtu otázok na jeden scenár.
Vo všeobecnosti platí, že za menej otázok dostanete viac bodov.

Nech $Q$ označuje priemerný počet otázok na scenár použitých vaším programom (pre všetky scenáre), zaokrúhlený na jedno desatinné miesto nadol.
Pre každý vstup je určená hodnota $F$ (viď ďalej).
Body pre daný vstup sa vypočítajú podľa nasledujúcich pravidiel:

- ak $Q > 10 \cdot F$, dostanete 0 bodov.
- ak $F < Q \leq 10 \cdot F$, počet bodov sa určí podľa nasledujúceho vzorca:
  $$ 90 \cdot \frac{F}{F + 4 \cdot (Q-F)} $$
- ak $Q \leq F$, dostanete plných 90 bodov.

Vaše riešenie bude hodnotené na niekoľkých vstupoch s rôznymi hodnotami $P$.
Celkový počet bodov, ktoré dostanete, bude
rovný minimálnemu počtu bodov vo všetkých vstupoch (t.j. naprieč všetkými pravdepodobnosťami $P$).

Hodnotiť sa bude na nasledovných vstupoch:

| $P$      | $F$      |
|----------|---------:|
|0.001     |      15.1|
|0.005256  |      51.1|
|0.011546  |      94.9|
|0.028545  |     191.5|
|0.039856  |     246.3|
|0.068648  |     366.2|
|0.104571  |     490.3|
|0.158765  |     639.1|
|0.2       |     731.4|

Hodnotiaci systém poskytne spätnú väzbu pre každý vstup.
Táto spätná väzba bude obsahovať hodnotu $Q$ vášho riešenia pre každý vstup, v ktorom ste získali nenulový počet bodov.

### Ukážka interakcie

Tu je ukážka interakcie s hodnotiacim systémom.
Upozorňujeme, že hodnoty $N$ a $T$ sa vyskutujú iba raz na začiatku interakcie.
Nezabudnite po každom riadku spláchnuť výstup.

| Váš vstup | Váš výstup |
|------------|----------------|
| `10 0.4 2` |                |
|            | `Q 1000000000` |
| `P`        |                |
|            | `Q 0000001000` |
| `P`        |                |
|            | `Q 0000000001` |
| `P`        |                |
|            | `Q 0111110110` |
| `N`        |                |
|            | `A 1000001001` |
| `C`        |                |
|            | `A 0000000000` |
| `W`        | &nbsp;         |

Program vyriešil prvý scenár správne,
ale druhý nie, pretože správne riešenie bolo `1100010010`
(ktoré program nemohol poznať, pretože sa nespýtal žiadne otázky).
Aj keby na vstupe bol ďalší scenár, program by mal okamžite skončiť.
